;
;****************************************************************************
;
;  Purpose:
;	Drivers For A Xicor 24C16 Or Compatible Serial NVRAM
;
;  Date:
;	04/30/92
;
;  Author:
;	John C. Wren
;
;  Modications:
;	02/04/97 - Added Description Fields For Archive
;
;  Processor:
;	Generic 8031
;
;  Assembler:
;	Avocet AVA51
;
;  Dependencies:
;	None
;
;  Files:
;	X24C16IO.ASM
;
;  Comments:
;	Be Sure To Check The DELAY Routine At The Bottom.  The 24C16 Has
;	Certain Timing Restrictions, In That Certain Things Cannot Be Done
;	Too Quickly.  The Delay At The Bottom Is Fine For A Generic 8031
;	Running At 12MHz.  If You're Using A Dallas Part, Or A 8031 Running
;	At A High Clock Speed, You'll Need To Adjust The Routine To Waste
;	About 6 Microseconds.
;
;  Philosophic:
;	These Drivers Should Support Many Of The Smaller Parts, As Long As
;	Care Is Taken Not To Write Into Addresses That Don't Exist.  
;	
;	One Change That Should Be Made To These Routines Is While The Code
;	Is Waiting For The NVRAM To Return Ready, If The NVRAM Has Failed
;	Internally, The Routines Will Wait Forever.  I Had An Implementation
;	That Used A Timer To Set A Timeout, But It Was Too Complex For A
;	Generic Routine Post To The Archive. 
;
;	This Could Be Reimplented Fairly Easily By Checking A Bit Variable
;	In The Loop That Waits For The NVRAM Busy.  Before Starting The
;	Wait, Clear The Bit.  In The Timer Routine, Setup A Handler That
;	Will Set The Bit Every 100ms Or So.  If The Wait Routine Sees The
;	Timeout Bit Get Set, Return An Error.
;
;	In Addition, No Check Is Done To See That Bytes Were Written 
;	Correctly.  I Consider That To Be A Upper Layer Process, As These
;	Are Just Low Level Read/Write Routines.
;
;	There Is An Extra Layer Of I/O Overhead, In That Certain Routines
;	Call A Function To Set A Line Low, Which Then Call The Actual
;	Hardware Drivers In The X24C16IO.ASM.  These Could Be Removed,
;	If Desired.  They Were Originally Here Because The Code Supported
;	Two NVRAMs, One On A Backplane, And One On A CPU Card.  A Entry
;	Point Was Provided To Select Which NVRAM Was Being Used, And The
;	First Layer Of I/O Determined Which Set Of I/O Handlers To Call.
;	If Stack Or Code Space Or Ultimate Speed Are Needed, Remove The
;	Extra Layer.
;
;****************************************************************************
;
;  Includes
;
;		%include "equates.inc"
		seg	code
;
;****************************************************************************
;
;  Publics
;
		public	X24C16_INIT
		public	X24C16_READ
		public	X24C16_WRITE
;
		extrn	IO_24C16SDA_RD
		extrn	IO_24C16SDA_LO
		extrn	IO_24C16SDA_HI
		extrn	IO_24C16SCL_LO
		extrn	IO_24C16SCL_HI
;
;****************************************************************************
;
;  Description:
;	Initialize NVRAM Control Lines
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Must Be Called Before Any Calls To Read/Write Functions Are Performed
;
X24C16_INIT	proc
		call	X24C16_SCL_LOW		; Set NVRAM Clock Low
		call	X24C16_SDA_HIGH		; Set NVRAM Data Line High
		ret				; Return To Caller	
		endproc
;
;****************************************************************************
;
;  Description:
;	Read Byte From EEPROM 
;
;  Entry Requirements:
;	DPTR Points To A Valid Location In NVRAM
;
;  On Exit:
;	Acc Has Byte Read From NVRAM
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_READ	proc
		clr	c			; Specify Write Mode For Dummy Write
		call	X24C16_START		; Send Start Sequence
		call	X24C16_ADRHI		; Write High Address To EEPROM
		call	X24C16_ADRLO		; Write Low Address To EEPROM
		setb	c			; Specify Read Mode
		call	X24C16_START		; Send Start Sequence To EEPROM
		call	X24C16_ADRHI		; Write High Address To EEPROM
		call	X24C16_IDATA		; Get Upper Byte Of Data
		call	X24C16_STOP		; Send Stop Bit
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Write Byte To EEPROM
;
;  Entry Requirements:
;	DPTR Contains Valid Address In NVRAM To Write Byte To
;	Acc Contains Byte To Write At DPTR
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_WRITE 	proc
		push	acc			; Save Value To Write
;
l?p1		call	X24C16_WAIT		; Is Part Busy?
		jc	l?p1			; If Busy, Loop
;
		call	X24C16_ADRLO		; Write Low Address To EEPROM
		pop	acc			; Recover Value To Write
		push	acc			; Save It Back
		call	X24C16_ODATA		; Send Data
		call	X24C16_STOP		; Send Stop Bit
		call	X24C16_WAIT		; Wait For NVRAM To Finish Up
		pop	acc			; Recover Callers Original Value
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Wait For NVRAM To Complete Operation
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Flawed In That If Part Never Responds, Routine Will Hang.  Should 
;	Add Some Kind Of Timer Protection Here.
;
X24C16_WAIT	proc
		clr	c			; Specify Write Mode
		call	X24C16_START		; Send Start Sequence To EEPROM
		call	X24C16_ADRHI		; Write High Address To EEPROM
		jc	X24C16_WAIT		; If Device Not ACK'ing, Is It Busy?
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Sends Top 3 Bits Of Starting Address Plus Mode To EEPROM.  
;
;  Entry Requirements:
;	DPH Contains Upper 3 Bits Of Starting Address To Send.
;	CY == 1 For Read, CY == 0 For Write
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_ADRHI 	proc
		push	b			; Save B
		clr	a			; Clear Acc
		mov	acc.0,c			; Set Mode Into Low Order Bit
		mov	b,a			; Store In B
		mov	a,dph			; Get High Byte Of Address
		rl	a			; Shift 9th..11th Bits
		anl	a,#00001110b		; Keep Only Those Bits
		orl	a,b			; OR In Mode
		orl	a,#0a0h			; OR In Device Identifier
		call	X24C16_ODATA		; Write To EEPROM
		pop	b			; Recover B
		ret				; Return To Caller
;
;****************************************************************************
;
;  Description:
;	Sends Lower 8 Bits Of Starting Address Plus Mode To EEPROM.  
;
;  Entry Requirements:
;	DPL Contains Low 8 Bits Starting Address To Send.
;	CY == 1 For Read, CY == 0 For Write
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_ADRLO 	mov	a,dpl			; Get Low Address
		call	X24C16_ODATA		; Write To EEPROM
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Sends Start Sequence To EEPROM
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_START 	proc
		push	psw			; Save R/W Mode
		call	X24C16_SDA_HIGH		; Set Data Line High
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SCL_HIGH		; Set Clock Line High
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SDA_LOW		; Set Data Line Low
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SCL_LOW		; Set Clock Line Low
		call	X24C16_DELAY		; Waste About 7 us
		pop	psw			; Recover R/W Mode
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Sends Stop Sequence To EEPROM
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_STOP	proc
		call	X24C16_SCL_LOW		; Set Clock Line Low
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SDA_LOW		; Set Data Line Low
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SCL_HIGH		; Set Clock Line High
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SDA_HIGH		; Set Data Line High
		call	X24C16_DELAY		; Waste About 7 us
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Read A Byte Of Data From EEPROM
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	Acc Has Byte Read
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Address & Mode Must Have Already Been Sent To NVRAM, And NVRAM Is
;	Expecting To Have Data Read From It.
;
X24C16_IDATA 	proc
		push	0			; Save R0
		mov	r0,#8			; Number Of Bits In A Byte
;
l?p2		call	X24C16_SCL_HIGH		; Set Clock Line High
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SDA_READ		; Get Data Line Status
		rlc	a			; Shift Into A
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SCL_LOW		; Set Clock Line Low
		call	X24C16_DELAY		; Waste About 7 us
		djnz	r0,l?p2			; Repeat For All 8 Bytes
;
		pop	0			; Recover R0
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Outputs Data In Acc To EEPROM.  
;
;  Entry Requirements:
;	Acc Has Byte To Write To NVRAM
;
;  On Exit:
;	CY == 0 If Device ACK'ed, CY == 1 If Device Failed To ACK.
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	5 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Used To Write Address And Mode To NVRAM, And Also Actual Data To 
;	Be Stored.
;
X24C16_ODATA 	proc
		push	0			; Save R0
		push	acc			; Save In Case We Use It
		mov	r0,#8			; Number Bits In A Byte
;
l?p1		rlc	a			; Shift Msb C Into Carry
		call	X24C16_SDA_WRITE	; Set NVRAM Data Line According To Bit
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SCL_HIGH		; Set Clock Line High
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SCL_LOW		; Set Clock Line Low
		call	X24C16_DELAY		; Waste About 7 us
		djnz	r0,l?p1			; If All 8 Bits Not Sent, Continue
;
		call	X24C16_SDA_HIGH		; Drive High For Tristate
;
l?p2		call	X24C16_SCL_HIGH		; Set Clock Line High
		call	X24C16_DELAY		; Waste About 7 us
		call	X24C16_SDA_READ		; Get Data Line Status
		push	psw			; Save CY
		call	X24C16_SCL_LOW		; Set Clock Line Back Low
		call	X24C16_DELAY		; Waste About 7 us
;
l?p3		pop	psw			; Recover ACK State
		pop	acc			; Recover Value Read
		pop	0			; Recover R0
		ret				; Return To Caller 
		endproc
;
;****************************************************************************
;
;  Description:
;	Calls Hardware Routine To Read SDA Pin
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY Has Data Bit Read From NVRAM
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_SDA_READ	proc
		call	IO_24C16SDA_RD		; Read Data Line
		ret				; Return To Caller 
		endproc
;
;****************************************************************************
;
;  Description:
;	Calls Hardware Routine To Set SDA Pin High Or Low, Based On CY
;
;  Entry Requirements:
;	CY Has State To Set SDA Pin To
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_SDA_WRITE proc
		jc	l?p1			; If Set High, Skip
		call	X24C16_SDA_LOW		; Set Data Line Low
		ret				; Return To Caller
;
l?p1		call	X24C16_SDA_HIGH		; Set Data Line High
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Calls Hardware Routine To Set SCL Line High
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_SCL_HIGH	proc
		call	IO_24C16SCL_HI		; Clock Line High
		ret				; Return To Caller 
		endproc
;
;****************************************************************************
;
;  Description:
;	Calls Hardware Routine To Set SCL Line Low
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_SCL_LOW	proc
		call	IO_24C16SCL_LO		; Clock Line Low
		ret				; Return To Caller 
		endproc
;
;****************************************************************************
;
;  Description:
;	Calls Hardware Routine To Set SDA Line High
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_SDA_HIGH	proc
		call	IO_24C16SDA_HI		; Data Line High
		ret				; Return To Caller 
		endproc
;
;****************************************************************************
;
;  Description:
;	Calls Hardware Routine To Set SDA Line Low
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
X24C16_SDA_LOW	proc
		call	IO_24C16SDA_LO		; Data Line Low
		ret				; Return To Caller 
		endproc
;
;****************************************************************************
;
;  Description:
;	Waste At Least 6us
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	0 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	This Is Processor Clock Sensitive.  It's Too Short A Time To Really
;	Implement As A Timer Function.  On A Generic 8031 Running At 12MHz,
;	This Can Simple Be 6 NOPs.  However, On A Dallas 80C320 Running At 
;	24MHz, This Would Need To Be A Lot Of NOPs.  
;
X24C16_DELAY	proc
		nop				; 1us @ 12MHz
		nop				; 1us @ 12MHz
		nop				; 1us @ 12MHz
		nop				; 1us @ 12MHz
		nop				; 1us @ 12MHz
		nop				; 1us @ 12MHz
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
		end
